@@ -27,6 +27,7 @@
 #include "gmessages.h"
 #include "gstrfuncs.h"
 #include "gthread.h"
+#include "gthreadprivate.h"
 #ifdef G_OS_WIN32
 #include "gwin32.h"
 #endif
@@ -187,10 +188,7 @@ g_get_charset (const char **charset)
   const gchar *raw;
 
   if (!cache)
-    {
-      cache = g_new0 (GCharsetCache, 1);
-      g_private_set (&cache_private, cache);
-    }
+    cache = g_private_set_alloc0 (&cache_private, sizeof (GCharsetCache));
 
   G_LOCK (aliases);
   raw = _g_locale_charset_raw ();
