@@ -38,7 +38,6 @@
 #include "gtestutils.h"
 #include "gslice.h"
 #include "grefcount.h"
-#include "gvalgrind.h"
 
 /* The following #pragma is here so we can do this...
  *
@@ -247,7 +246,7 @@
 
 struct _GHashTable
 {
-  gint             size;
+  gsize            size;
   gint             mod;
   guint            mask;
   gint             nnodes;
@@ -280,9 +279,9 @@ typedef struct
   GHashTable  *hash_table;
   gpointer     dummy1;
   gpointer     dummy2;
-  int          position;
+  gint         position;
   gboolean     dummy3;
-  int          version;
+  gint         version;
 } RealIter;
 
 G_STATIC_ASSERT (sizeof (GHashTableIter) == sizeof (RealIter));
@@ -555,59 +554,16 @@ g_hash_table_remove_node (GHashTable   *hash_table,
 
 }
 
-/*
- * g_hash_table_setup_storage:
- * @hash_table: our #GHashTable
- *
- * Initialise the hash table size, mask, mod, and arrays.
- */
-static void
-g_hash_table_setup_storage (GHashTable *hash_table)
-{
-  gboolean small;
-
-  /* We want to use small arrays only if:
-   *   - we are running on a system where that makes sense (64 bit); and
-   *   - we are not running under valgrind.
-   */
-  small = FALSE;
-
-#ifdef USE_SMALL_ARRAYS
-  small = TRUE;
-
-# ifdef ENABLE_VALGRIND
-  if (RUNNING_ON_VALGRIND)
-    small = FALSE;
-# endif
-#endif
-
-  g_hash_table_set_shift (hash_table, HASH_TABLE_MIN_SHIFT);
-
-  hash_table->have_big_keys = !small;
-  hash_table->have_big_values = !small;
-
-  hash_table->keys   = g_hash_table_realloc_key_or_value_array (NULL, hash_table->size, hash_table->have_big_keys);
-  hash_table->values = hash_table->keys;
-  hash_table->hashes = g_new0 (guint, hash_table->size);
-}
-
 /*
  * g_hash_table_remove_all_nodes:
  * @hash_table: our #GHashTable
  * @notify: %TRUE if the destroy notify handlers are to be called
  *
- * Removes all nodes from the table.
+ * Removes all nodes from the table.  Since this may be a precursor to
+ * freeing the table entirely, no resize is performed.
  *
  * If @notify is %TRUE then the destroy notify functions are called
  * for the key and value of the hash node.
- *
- * Since this may be a precursor to freeing the table entirely, we'd
- * ideally perform no resize, and we can indeed avoid that in some
- * cases.  However: in the case that we'll be making callbacks to user
- * code (via destroy notifies) we need to consider that the user code
- * might call back into the table again.  In this case, we setup a new
- * set of arrays so that any callers will see an empty (but valid)
- * table.
  */
 static void
 g_hash_table_remove_all_nodes (GHashTable *hash_table,
@@ -621,8 +577,6 @@ g_hash_table_remove_all_nodes (GHashTable *hash_table,
   gpointer *old_keys;
   gpointer *old_values;
   guint    *old_hashes;
-  gboolean  old_have_big_keys;
-  gboolean  old_have_big_values;
 
   /* If the hash table is already empty, there is nothing to be done. */
   if (hash_table->nnodes == 0)
@@ -631,7 +585,6 @@ g_hash_table_remove_all_nodes (GHashTable *hash_table,
   hash_table->nnodes = 0;
   hash_table->noccupied = 0;
 
-  /* Easy case: no callbacks, so we just zero out the arrays */
   if (!notify ||
       (hash_table->key_destroy_func == NULL &&
        hash_table->value_destroy_func == NULL))
@@ -652,52 +605,43 @@ g_hash_table_remove_all_nodes (GHashTable *hash_table,
       return;
     }
 
-  /* Hard case: we need to do user callbacks.  There are two
-   * possibilities here:
-   *
-   *   1) there are no outstanding references on the table and therefore
-   *   nobody should be calling into it again (destroying == true)
-   *
-   *   2) there are outstanding references, and there may be future
-   *   calls into the table, either after we return, or from the destroy
-   *   notifies that we're about to do (destroying == false)
-   *
-   * We handle both cases by taking the current state of the table into
-   * local variables and replacing it with something else: in the "no
-   * outstanding references" cases we replace it with a bunch of
-   * null/zero values so that any access to the table will fail.  In the
-   * "may receive future calls" case, we reinitialise the struct to
-   * appear like a newly-created empty table.
-   *
-   * In both cases, we take over the references for the current state,
-   * freeing them below.
-   */
+  /* Keep the old storage space around to iterate over it. */
   old_size = hash_table->size;
-  old_have_big_keys = hash_table->have_big_keys;
-  old_have_big_values = hash_table->have_big_values;
-  old_keys   = g_steal_pointer (&hash_table->keys);
-  old_values = g_steal_pointer (&hash_table->values);
-  old_hashes = g_steal_pointer (&hash_table->hashes);
+  old_keys   = hash_table->keys;
+  old_values = hash_table->values;
+  old_hashes = hash_table->hashes;
 
+  /* Now create a new storage space; If the table is destroyed we can use the
+   * shortcut of not creating a new storage. This saves the allocation at the
+   * cost of not allowing any recursive access.
+   * However, the application doesn't own any reference anymore, so access
+   * is not allowed. If accesses are done, then either an assert or crash
+   * *will* happen. */
+  g_hash_table_set_shift (hash_table, HASH_TABLE_MIN_SHIFT);
   if (!destruction)
-    /* Any accesses will see an empty table */
-    g_hash_table_setup_storage (hash_table);
+    {
+      hash_table->keys   = g_hash_table_realloc_key_or_value_array (NULL, hash_table->size, FALSE);
+      hash_table->values = hash_table->keys;
+      hash_table->hashes = g_new0 (guint, hash_table->size);
+    }
   else
-    /* Will cause a quick crash on any attempted access */
-    hash_table->size = hash_table->mod = hash_table->mask = 0;
+    {
+      hash_table->keys   = NULL;
+      hash_table->values = NULL;
+      hash_table->hashes = NULL;
+    }
 
-  /* Now do the actual destroy notifies */
   for (i = 0; i < old_size; i++)
     {
       if (HASH_IS_REAL (old_hashes[i]))
         {
-          key = g_hash_table_fetch_key_or_value (old_keys, i, old_have_big_keys);
-          value = g_hash_table_fetch_key_or_value (old_values, i, old_have_big_values);
+          key = g_hash_table_fetch_key_or_value (old_keys, i, hash_table->have_big_keys);
+          value = g_hash_table_fetch_key_or_value (old_values, i, hash_table->have_big_values);
 
           old_hashes[i] = UNUSED_HASH_VALUE;
 
-          g_hash_table_assign_key_or_value (old_keys, i, old_have_big_keys, NULL);
-          g_hash_table_assign_key_or_value (old_values, i, old_have_big_values, NULL);
+          g_hash_table_assign_key_or_value (old_keys, i, hash_table->have_big_keys, NULL);
+          g_hash_table_assign_key_or_value (old_values, i, hash_table->have_big_values, NULL);
 
           if (hash_table->key_destroy_func != NULL)
             hash_table->key_destroy_func (key);
@@ -707,6 +651,9 @@ g_hash_table_remove_all_nodes (GHashTable *hash_table,
         }
     }
 
+  hash_table->have_big_keys = FALSE;
+  hash_table->have_big_values = FALSE;
+
   /* Destroy old storage space. */
   if (old_keys != old_values)
     g_free (old_values);
@@ -851,7 +798,7 @@ static void
 g_hash_table_resize (GHashTable *hash_table)
 {
   guint32 *reallocated_buckets_bitmap;
-  guint old_size;
+  gsize old_size;
   gboolean is_a_set;
 
   old_size = hash_table->size;
@@ -1069,6 +1016,7 @@ g_hash_table_new_full (GHashFunc      hash_func,
   GHashTable *hash_table;
 
   hash_table = g_slice_new (GHashTable);
+  g_hash_table_set_shift (hash_table, HASH_TABLE_MIN_SHIFT);
   g_atomic_ref_count_init (&hash_table->ref_count);
   hash_table->nnodes             = 0;
   hash_table->noccupied          = 0;
@@ -1079,8 +1027,17 @@ g_hash_table_new_full (GHashFunc      hash_func,
 #endif
   hash_table->key_destroy_func   = key_destroy_func;
   hash_table->value_destroy_func = value_destroy_func;
+  hash_table->keys               = g_hash_table_realloc_key_or_value_array (NULL, hash_table->size, FALSE);
+  hash_table->values             = hash_table->keys;
+  hash_table->hashes             = g_new0 (guint, hash_table->size);
 
-  g_hash_table_setup_storage (hash_table);
+#ifdef USE_SMALL_ARRAYS
+  hash_table->have_big_keys = FALSE;
+  hash_table->have_big_values = FALSE;
+#else
+  hash_table->have_big_keys = TRUE;
+  hash_table->have_big_values = TRUE;
+#endif
 
   return hash_table;
 }
@@ -1148,14 +1105,14 @@ g_hash_table_iter_next (GHashTableIter *iter,
 #ifndef G_DISABLE_ASSERT
   g_return_val_if_fail (ri->version == ri->hash_table->version, FALSE);
 #endif
-  g_return_val_if_fail (ri->position < ri->hash_table->size, FALSE);
+  g_return_val_if_fail (ri->position < (gssize) ri->hash_table->size, FALSE);
 
   position = ri->position;
 
   do
     {
       position++;
-      if (position >= ri->hash_table->size)
+      if (position >= (gssize) ri->hash_table->size)
         {
           ri->position = position;
           return FALSE;
@@ -1198,7 +1155,7 @@ iter_remove_or_steal (RealIter *ri, gboolean notify)
   g_return_if_fail (ri->version == ri->hash_table->version);
 #endif
   g_return_if_fail (ri->position >= 0);
-  g_return_if_fail (ri->position < ri->hash_table->size);
+  g_return_if_fail ((gsize) ri->position < ri->hash_table->size);
 
   g_hash_table_remove_node (ri->hash_table, ri->position, notify);
 
@@ -1382,7 +1339,7 @@ g_hash_table_iter_replace (GHashTableIter *iter,
   g_return_if_fail (ri->version == ri->hash_table->version);
 #endif
   g_return_if_fail (ri->position >= 0);
-  g_return_if_fail (ri->position < ri->hash_table->size);
+  g_return_if_fail ((gsize) ri->position < ri->hash_table->size);
 
   node_hash = ri->hash_table->hashes[ri->position];
 
@@ -1923,7 +1880,7 @@ g_hash_table_foreach_remove_or_steal (GHashTable *hash_table,
                                       gboolean    notify)
 {
   guint deleted = 0;
-  gint i;
+  gsize i;
 #ifndef G_DISABLE_ASSERT
   gint version = hash_table->version;
 #endif
@@ -2032,7 +1989,7 @@ g_hash_table_foreach (GHashTable *hash_table,
                       GHFunc      func,
                       gpointer    user_data)
 {
-  gint i;
+  gsize i;
 #ifndef G_DISABLE_ASSERT
   gint version;
 #endif
@@ -2090,7 +2047,7 @@ g_hash_table_find (GHashTable *hash_table,
                    GHRFunc     predicate,
                    gpointer    user_data)
 {
-  gint i;
+  gsize i;
 #ifndef G_DISABLE_ASSERT
   gint version;
 #endif
@@ -2162,7 +2119,7 @@ g_hash_table_size (GHashTable *hash_table)
 GList *
 g_hash_table_get_keys (GHashTable *hash_table)
 {
-  gint i;
+  gsize i;
   GList *retval;
 
   g_return_val_if_fail (hash_table != NULL, NULL);
@@ -2210,7 +2167,7 @@ g_hash_table_get_keys_as_array (GHashTable *hash_table,
                                 guint      *length)
 {
   gpointer *result;
-  guint i, j = 0;
+  gsize i, j = 0;
 
   result = g_new (gpointer, hash_table->nnodes + 1);
   for (i = 0; i < hash_table->size; i++)
@@ -2248,7 +2205,7 @@ g_hash_table_get_keys_as_array (GHashTable *hash_table,
 GList *
 g_hash_table_get_values (GHashTable *hash_table)
 {
-  gint i;
+  gsize i;
   GList *retval;
 
   g_return_val_if_fail (hash_table != NULL, NULL);
