@@ -2,17 +2,17 @@
  * Copyright (C) 2011 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GLIB_PRIVATE_H__
@@ -20,12 +20,7 @@
 
 #include <glib.h>
 #include "gwakeup.h"
-
-#if defined(__GNUC__)
-# define _g_alignof(type) (__alignof__ (type))
-#else
-# define _g_alignof(type) (G_STRUCT_OFFSET (struct { char a; type b; }, b))
-#endif
+#include "gstdioprivate.h"
 
 GMainContext *          g_get_worker_context            (void);
 gboolean                g_check_setuid                  (void);
@@ -64,6 +59,25 @@ typedef struct {
   /* See glib-init.c */
   void                  (* glib_init)                   (void);
 
+  /* See gstdio.c */
+#ifdef G_OS_WIN32
+  int                   (* g_win32_stat_utf8)           (const gchar        *filename,
+                                                         GWin32PrivateStat  *buf);
+
+  int                   (* g_win32_lstat_utf8)          (const gchar        *filename,
+                                                         GWin32PrivateStat  *buf);
+
+  int                   (* g_win32_readlink_utf8)       (const gchar        *filename,
+                                                         gchar              *buf,
+                                                         gsize               buf_size,
+                                                         gchar             **alloc_buf,
+                                                         gboolean            terminate);
+
+  int                   (* g_win32_fstat)               (int                 fd,
+                                                         GWin32PrivateStat  *buf);
+#endif
+
+
   /* Add other private functions here, initialize them in glib-private.c */
 } GLibPrivateVTable;
 
