@@ -205,6 +205,26 @@
  */
 #define GLIB_VERSION_2_58       (G_ENCODE_VERSION (2, 58))
 
+/**
+ * GLIB_VERSION_2_60:
+ *
+ * A macro that evaluates to the 2.60 version of GLib, in a format
+ * that can be used by the C pre-processor.
+ *
+ * Since: 2.60
+ */
+#define GLIB_VERSION_2_60       (G_ENCODE_VERSION (2, 60))
+
+/**
+ * GLIB_VERSION_2_62:
+ *
+ * A macro that evaluates to the 2.62 version of GLib, in a format
+ * that can be used by the C pre-processor.
+ *
+ * Since: 2.62
+ */
+#define GLIB_VERSION_2_62       (G_ENCODE_VERSION (2, 62))
+
 /* evaluates to the current stable version; for development cycles,
  * this means the next stable target
  */
@@ -534,4 +554,32 @@
 # define GLIB_AVAILABLE_IN_2_58                 _GLIB_EXTERN
 #endif
 
+#if GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_60
+# define GLIB_DEPRECATED_IN_2_60                GLIB_DEPRECATED
+# define GLIB_DEPRECATED_IN_2_60_FOR(f)         GLIB_DEPRECATED_FOR(f)
+#else
+# define GLIB_DEPRECATED_IN_2_60                _GLIB_EXTERN
+# define GLIB_DEPRECATED_IN_2_60_FOR(f)         _GLIB_EXTERN
+#endif
+
+#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_60
+# define GLIB_AVAILABLE_IN_2_60                 GLIB_UNAVAILABLE(2, 60)
+#else
+# define GLIB_AVAILABLE_IN_2_60                 _GLIB_EXTERN
+#endif
+
+#if GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_62
+# define GLIB_DEPRECATED_IN_2_62                GLIB_DEPRECATED
+# define GLIB_DEPRECATED_IN_2_62_FOR(f)         GLIB_DEPRECATED_FOR(f)
+#else
+# define GLIB_DEPRECATED_IN_2_62                _GLIB_EXTERN
+# define GLIB_DEPRECATED_IN_2_62_FOR(f)         _GLIB_EXTERN
+#endif
+
+#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_62
+# define GLIB_AVAILABLE_IN_2_62                 GLIB_UNAVAILABLE(2, 62)
+#else
+# define GLIB_AVAILABLE_IN_2_62                 _GLIB_EXTERN
+#endif
+
 #endif /*  __G_VERSION_MACROS_H__ */
